@@ -41,76 +41,92 @@ class BayesBridge():
     # TODO: write a test to ensure that the output when resuming the Gibbs
     # sampler coincide with that without interruption.
     def gibbs_additional_iter(
-            self, mcmc_output, n_add_iter, n_status_update=0,
-            merge=False, deallocate=False):
+            self, prev_mcmc_info, n_add_iter, n_status_update=0,
+            merge=False, prev_samples=None):
         """ Resume Gibbs sampler from the last state.
 
         Parameter
         ---------
-        mcmc_output : dict
-            Output of a previous call to the 'gibbs' method.
+        prev_mcmc_info : dict
+            MCMC info returned by a previous call to the `gibbs` method.
         n_iter : int
         n_status_update : int
         merge : bool
             If True, merge the Gibbs sampler outputs from the previous and
-            current runs and then return.
-        deallocate : bool
-            If True, clear the samples from the previous Gibbs run to save
-            memory.
+            new runs and then return.
+        prev_samples : dict
+            MCMC samples returned by a previous call to the 'gibbs' method.
 
         Returns
         -------
-        next_mcmc_output : dict
+        new_samples : dict
+        new_mcmc_info : dict
         """
 
-        if merge and deallocate:
-            warn(
-                "To merge the outputs, the previous one cannot be deallocated.")
-            deallocate = False
+        if merge and prev_samples is None:
+            raise ValueError(
+                "To merge the outputs from previous and new MCMC runs, you "
+                "have to supply the optional argument `prev_samples`."
+            )
 
-        self.rg.set_state(mcmc_output['_random_gen_state'])
+        self.rg.set_state(prev_mcmc_info['_random_gen_state'])
 
-        init = mcmc_output['_markov_chain_state']
+        init = prev_mcmc_info['_markov_chain_state']
         thin, bridge_exp, coef_sampler_type = (
-            mcmc_output[key]
+            prev_mcmc_info[key]
             for key in ['thin', 'bridge_exponent', 'coef_sampler_type']
         )
-        params_to_save = mcmc_output['samples'].keys()
+        curvature_est_stabilized = prev_mcmc_info['options']['hmc_curvature_est_stabilized']
+        params_to_save = prev_mcmc_info['saved_params']
 
         # Initalize the regression coefficient sampler with the previous state.
         self.reg_coef_sampler = SparseRegressionCoefficientSampler(
-            self.n_pred, self.prior_sd_for_unshrunk, coef_sampler_type
+            self.n_pred, self.prior_sd_for_unshrunk, coef_sampler_type,
+            curvature_est_stabilized, self.prior.slab_size
         )
-        self.reg_coef_sampler.set_internal_state(mcmc_output['_reg_coef_sampler_state'])
+        self.reg_coef_sampler.set_internal_state(prev_mcmc_info['_reg_coef_sampler_state'])
 
-        if deallocate:
-            mcmc_output['samples'].clear()
-
-        next_mcmc_output = self.gibbs(
+        new_samples, new_mcmc_info = self.gibbs(
             n_add_iter, 0, thin, init=init,
             params_to_save=params_to_save,
             n_status_update=n_status_update,
-            options=mcmc_output['options'],
+            options=prev_mcmc_info['options'],
             _add_iter_mode=True
         )
         if merge:
-            next_mcmc_output \
-                = self.manager.merge_outputs(mcmc_output, next_mcmc_output)
+            new_samples, new_mcmc_info = self.manager.merge_outputs(
+                prev_samples, prev_mcmc_info, new_samples, new_mcmc_info
+            )
 
-        return next_mcmc_output
+        return new_samples, new_mcmc_info
 
     def gibbs(self, n_iter, n_burnin=0, thin=1, seed=None,
-              init={}, params_to_save=('coef', 'global_scale', 'logp'),
-              coef_sampler_type=None, n_init_optim=10, n_status_update=0,
+              init={'global_scale': 0.1}, params_to_save=('coef', 'global_scale', 'logp'),
+              coef_sampler_type=None, n_status_update=0,
               options=None, _add_iter_mode=False):
-        """ Sample from the posterior under the specified model and prior.
+        """ Generate posterior samples under the specified model and prior.
 
         Parameters
         ----------
         n_iter : int
-            total number of MCMC iterations i.e. burn-ins + saved posterior draws
+            Total number of MCMC iterations i.e. burn-ins + saved posterior draws
         n_burnin : int
-            number of burn-in samples to be discarded
+            Number of burn-in samples to be discarded
+        thin : int
+            Number of iterations per saved samples for "thinning" MCMC to reduce
+            the output size. In other words, the function saves an MCMC sample
+            every `thin` iterations, returning floor(n_iter / thin) samples.
+        seed : int
+            Seed for random number generator.
+        init : dict of numpy arrays
+            Specifies, partially or completely, the initial state of Markov chain.
+            The partial option allows either specifying the global scale or
+            regression coefficients. The former is the recommended default option
+            since the global scale parameter is easier to choose, representing
+            the prior expected magnitude of regression coefficients and . (But
+            the latter might make sense if some preliminary estimate of coefficients
+            are available.) Other parameters are then initialized through a
+            combination of heuristics and conditional optimization.
         coef_sampler_type : {None, 'cholesky', 'cg', 'hmc'}
             Specifies the sampling method used to update regression coefficients.
             If None, the method is chosen via a crude heuristic based on the
@@ -119,14 +135,8 @@ class BayesBridge():
             the conjugate gradient sampler ('cg') is preferred over the
             Cholesky decomposition based sampler ('cholesky'). For other
             models, only Hamiltonian Monte Carlo ('hmc') can be used.
-        n_init_optim : int
-            If > 0, the Markov chain will be run after the specified number of
-            optimization steps in which the regression coefficients are
-            optimized conditionally on the shrinkage parameters. During the
-            optimization, the global shrinkage parameter is fixed while the
-            local ones are sampled.
         params_to_save : {'all', tuple or list of str}
-            Specifies which parameters to save during MCMC iterations. If None,
+            Specifies which parameters to save during MCMC iterations. By default,
             the most relevant parameters --- regression coefficients,
             global scale, posterior log-density --- are saved. Use all to save
             all the parameters (but beaware of the extra memory requirement),
@@ -143,25 +153,23 @@ class BayesBridge():
 
         Returns
         -------
-        mcmc_output : dict
-            Contains posterior samples under the key 'samples,' along with the
-            sampler settings to reproduce and resume the sampling process.
-        samples = mcmc_output['samples'] : dict of numpy arrays
+        samples : dict
             Contains MCMC samples of the parameters as specified by
             **params_to_save**. The last dimension of the arrays correspond
             to MCMC iterations; for example,
             :code:`samples['coef'][:, 0]`
             is the first MCMC sample of regression coefficients.
+        mcmc_info : dict
+            Contains information on the MCMC run and sampler settings, enough in
+            particular to reproduce and resume the sampling process.
         """
 
         if not isinstance(options, SamplerOptions):
-            options = SamplerOptions.create(
+            options = SamplerOptions.pick_default_and_create(
                 coef_sampler_type, options, self.model.name, self.model.design
             )
 
-        if _add_iter_mode:
-            n_init_optim = 0
-        else:
+        if not _add_iter_mode:
             self.rg.set_seed(seed)
             self.reg_coef_sampler = SparseRegressionCoefficientSampler(
                 self.n_pred, self.prior_sd_for_unshrunk,
@@ -182,10 +190,7 @@ class BayesBridge():
 
         # Initial state of the Markov chain
         coef, obs_prec, lscale, gscale, init, initial_optim_info = \
-            self.initialize_chain(init, self.prior.bridge_exp, n_init_optim)
-        if n_init_optim > 0:
-            self.manager.print_status(
-                n_status_update, 0, n_iter, msg_type='optim', time_format='second')
+            self.initialize_chain(init, self.prior.bridge_exp)
 
         # Pre-allocate
         samples = {}
@@ -243,8 +248,7 @@ class BayesBridge():
             self.manager.pack_parameters(coef, obs_prec, lscale, gscale)
 
         _reg_coef_sampling_info = None
-        mcmc_output = {
-            'samples': samples,
+        mcmc_info = {
             'init': init,
             'n_iter': n_iter,
             'n_burnin': n_burnin,
@@ -254,20 +258,20 @@ class BayesBridge():
             'prior_sd_for_unshrunk': self.prior_sd_for_unshrunk,
             'bridge_exponent': self.prior.bridge_exp,
             'coef_sampler_type': options.coef_sampler_type,
+            'saved_params': params_to_save,
             'runtime': runtime,
             'options': options.get_info(),
-            'initial_optimization_info': initial_optim_info,
+            '_init_optim_info': initial_optim_info,
             '_reg_coef_sampling_info': sampling_info,
             '_markov_chain_state': _markov_chain_state,
             '_random_gen_state': self.rg.get_state(),
             '_reg_coef_sampler_state': self.reg_coef_sampler.get_internal_state()
         }
 
-        return mcmc_output
+        return samples, mcmc_info
 
-
-    def initialize_chain(self, init, bridge_exp, n_optim):
-        # Choose the user-specified state if provided, the default ones otherwise.
+    def initialize_chain(self, init, bridge_exp):
+        """ Choose the user-specified state if provided, the default ones otherwise."""
 
         valid_param_name \
             = ('coef', 'local_scale', 'global_scale', 'obs_prec', 'logp')
@@ -276,47 +280,63 @@ class BayesBridge():
                 warn("'{:s}' is not a valid parameter name and "
                      "will be ignored.".format(key))
 
+        coef_only_specified = 'coef' in init and ('global_scale' not in init)
+
         if 'coef' in init:
             coef = init['coef']
             if not len(coef) == self.n_pred:
-                raise ValueError('An invalid initial state.')
+                raise ValueError('Invalid initial length of regression coefficient.')
         else:
             coef = np.zeros(self.n_pred)
-            if 'intercept' in init:
-                coef[0] = init['intercept']
+            if self.model.name in ('linear', 'logit'):
+                coef[0] = self.model.calc_intercept_mle()
 
-        if 'obs_prec' in init:
-            obs_prec = np.ascontiguousarray(init['obs_prec'])
-                # Cython requires a C-contiguous array.
-            if not len(obs_prec) == self.n_obs:
-                raise ValueError('An invalid initial state.')
-        elif self.model.name == 'linear':
-            obs_prec = np.mean((self.model.y - self.model.design.dot(coef)) ** 2) ** -1
-        elif self.model.name == 'logit':
-            obs_prec = LogisticModel.compute_polya_gamma_mean(
-                self.model.n_trial, self.model.design.dot(coef)
+        obs_prec = self.initialize_obs_precision(init, coef)
+
+        if coef_only_specified:
+            gscale = self.update_global_scale(
+                None, coef[self.n_unshrunk:], bridge_exp,
+                method='optimize'
+            )
+            lscale = self.update_local_scale(
+                gscale, coef[self.n_unshrunk:], bridge_exp
             )
         else:
-            obs_prec = None
+            if 'global_scale' not in init:
+                raise ValueError("Initial global scale must be specified when "
+                                 "coefficients aren't specified.")
+            if self.prior._gscale_paramet == 'raw':
+                warn("Using the raw global scale parametrization; make sure that "
+                     "the specified initial value is scaled accordingly.")
+            gscale = init['global_scale']
+            if 'local_scale' in init:
+                lscale = init['local_scale']
+                if not len(lscale) == (self.n_pred - self.n_unshrunk):
+                    raise ValueError('Invalid initial length of local scale parameter')
+            else:
+                lscale = np.ones(self.n_pred - self.n_unshrunk) / gscale
 
-        lscale, gscale = self.initialize_shrinkage_parameters(init, bridge_exp)
+        if self.prior._gscale_paramet == 'coef_magnitude':
+            # Gibbs sampler requires the raw parametrization, though
+            # technically only gscale * lscale matters due to the update order.
+            gscale, lscale \
+                = self.prior.adjust_scale(gscale, lscale, to='raw')
 
-        info_keys = ['is_success', 'n_design_matvec', 'n_iter']
-        optim_info = {
-            key: np.zeros(n_optim, dtype=np.int) for key in info_keys
-        }
-        optim_info['n_optim'] = n_optim
-        for i in range(n_optim):
+        if 'coef' not in init:
+            # Optimize coefficients and then update other parameters
             coef, info = self.reg_coef_sampler.search_mode(
                 coef, lscale, gscale, obs_prec, self.model
             )
-            for key in info_keys:
-                optim_info[key][i] = info[key]
             obs_prec = self.update_obs_precision(coef)
             lscale = self.update_local_scale(
                 gscale, coef[self.n_unshrunk:], bridge_exp
             )
-            
+            optim_info = {
+                key: info[key] for key in ['is_success', 'n_design_matvec', 'n_iter']
+            }
+        else:
+            optim_info = None
+
         init = {
             'coef': coef,
             'obs_prec': obs_prec,
@@ -326,44 +346,22 @@ class BayesBridge():
 
         return coef, obs_prec, lscale, gscale, init, optim_info
 
-    def initialize_shrinkage_parameters(self, init, bridge_exp):
-        """
-        Current options allow specifying 1) both scale parameters directly,
-        2) regression coefficients only, and 3) global scale only.
-        """
-        gscale_default = .1
-        if self.prior._gscale_paramet == 'raw':
-            gscale_default \
-                /= self.prior.compute_power_exp_ave_magnitude(bridge_exp)
-
-        if 'local_scale' in init and 'global_scale' in init:
-            lscale = init['local_scale']
-            gscale = init['global_scale']
-            if not len(lscale) == (self.n_pred - self.n_unshrunk):
+    def initialize_obs_precision(self, init, coef):
+        if 'obs_prec' in init:
+            obs_prec = np.ascontiguousarray(init['obs_prec'])
+            # Cython requires a C-contiguous array.
+            if not len(obs_prec) == self.n_obs:
                 raise ValueError('An invalid initial state.')
-
-        elif 'coef' in init:
-            gscale = self.update_global_scale(
-                None, init['coef'][self.n_unshrunk:], bridge_exp,
-                method='optimize'
-            )
-            lscale = self.update_local_scale(
-                gscale, init['coef'][self.n_unshrunk:], bridge_exp
+        elif self.model.name == 'linear':
+            obs_prec = np.mean(
+                (self.model.y - self.model.design.dot(coef)) ** 2) ** -1
+        elif self.model.name == 'logit':
+            obs_prec = LogisticModel.compute_polya_gamma_mean(
+                self.model.n_trial, self.model.design.dot(coef)
             )
         else:
-            if 'global_scale' in init:
-                gscale = init['global_scale']
-            else:
-                gscale = gscale_default
-            lscale = np.ones(self.n_pred - self.n_unshrunk) / gscale
-
-        if self.prior._gscale_paramet == 'coef_magnitude':
-            # Gibbs sampler requires the raw parametrization. Technically only
-            # gscale * lscale matters within the sampler due to the update order.
-            gscale, lscale \
-                = self.prior.adjust_scale(gscale, lscale, to='raw')
-
-        return lscale, gscale
+            obs_prec = None
+        return obs_prec
 
     def update_regress_coef(self, coef, obs_prec, gscale, lscale, sampling_method):
 
