@@ -29,8 +29,18 @@ class LinearModel(AbstractModel):
         hessian_op = lambda v: - obs_prec * self.design.Tdot(self.design.dot(v))
         return hessian_op
 
+    def calc_intercept_mle(self):
+        return self.y.mean()
+
     @staticmethod
     def simulate_outcome(X, beta, noise_sd, seed=None):
-        np.random.seed(seed)
+        """
+        Parameters
+        ----------
+        X : DesignMatrix, numpy/scipy matrix
+            Only needs to support the `dot()` operation
+        """
+        if seed is not None:
+            np.random.seed(seed)
         y = X.dot(beta) + noise_sd * np.random.randn(X.shape[0])
         return y
\ No newline at end of file
